#include "2D.frag"

uniform float time;

float twopi = 6.283185307179586;
float smoothing = 1.0;

vec3 colours[5] = vec3[]
  ( vec3(1.0, 0.0, 0.0) // red
  , vec3(1.0, 0.8, 0.0) // yellow
  , vec3(0.27, 1.0, 0.0) // green
  , vec3(0.0, 0.53, 1.0) // blue
  , vec3(0.8, 0.0, 1.0) // purple
  );

float circle(vec2 z, vec2 c, float r) {
  float delta = length(vec4(dFdx(z), dFdy(z)));
  float smooth = smoothing * delta;
  return 1.0 - smoothstep(r - smooth, r + smooth, length(z - c));
}

float wave(vec2 z, vec2 c, float w, float a) {
  float delta = length(vec4(dFdx(z), dFdy(z)));
  float smooth = smoothing * delta;
  float y = sin(a - twopi / 16.0 * max(0.0, z.x - 1.25));
  return smoothstep(c.x - smooth, c.x + smooth, z.x) *
    smoothstep(y - w - smooth, y - w + smooth, z.y) *
   (1.0 - smoothstep(y + w - smooth, y + w + smooth, z.y));
}

float impulses(vec2 z, vec2 c, float i, float a) {
  float delta = length(vec4(dFdx(z), dFdy(z)));
  float smooth = smoothing * delta;
  float smoothx = smooth * twopi / 16.0;
  float x = a - twopi / 16.0 * (z.x - 1.25);
  float y = sin(x);
  float j = (y + 1.0) / 60.0;
  float j2 = (y + 1.0) / 180.0;
  float k = floor((x / twopi * 36.0) + 0.5) / 36.0 * twopi;
  return 
    smoothstep(-1.0 - 2.5 * i - smooth, -1.0 - 2.5 * i + smooth, z.y) *
   (1.0 - smoothstep(1.0 - 2.5 * i - smooth, 1.0 - 2.5 * i + smooth, z.y)) *
     smoothstep(k - j - smoothx, k - j + smoothx, x) * 
    (1.0 - smoothstep(k + j - smoothx, k + j + smoothx, x)) +
    smoothstep(-1.0 - 2.5 * 6.0 - smooth, -1.0 - 2.5 * 6.0 + smooth, z.y) *
   (1.0 - smoothstep(1.0 - 2.5 * 6.0 - smooth, 1.0 - 2.5 * 6.0 + smooth, z.y)) *
     smoothstep(k - j2 - smoothx, k - j2 + smoothx, x) * 
    (1.0 - smoothstep(k + j2 - smoothx, k + j2 + smoothx, x));
}

vec3 color(vec2 z) {
  z *= 9.0;
  z += vec2(1.0 + 7.5 * 16.0 / 9.0, -7.5);
  vec3 colour = vec3(1.0); // white

  colour = mix(colour, colour * vec3(0.0), circle(z, vec2(0.0), 1.01) - circle(z, vec2(0.0), 0.99));
  colour = mix(colour, colour * vec3(0.0), circle(z, vec2(0.0), 0.02));

  for (int i = 0; i < 5; ++i) {
     float a = twopi * (float(i) / 5.0 + fract(time / 5.0));
     vec2 c = vec2(cos(a), sin(a));
     float r = 0.125;
     float w = r / 4.0;
     colour = mix(colour, colour * colours[i], min(1.0, circle(z, c, r) + wave(z, c, w, a)));
     colour = mix(colour, colour * colours[i], impulses(z, c, float(i + 1.0), a));
  }

  return colour;
}
